@@ -106,67 +106,52 @@ static enum {
 	PROP_ALARM_0,
 	PROP_DIR,
 	PROP_ID,
+    PROP_TRIGGERED,
 	PROP_TYPE,
 	PROP_TIME,
 	PROP_TIMESTAMP,
 	PROP_ACTIVE,
 	PROP_MESSAGE,
 	PROP_REPEAT,
-	PROP_SNOOZE,
 	PROP_NOTIFY_TYPE,
 	PROP_SOUND_FILE,
 	PROP_SOUND_LOOP,
-	PROP_COMMAND,
-	PROP_NOTIFY_BUBBLE
+	PROP_COMMAND
 } AlarmProp;
 
 #define PROP_NAME_DIR			"gconf-dir"
 #define PROP_NAME_ID			"id"
+#define PROP_NAME_TRIGGERED     "triggered"
 #define PROP_NAME_TYPE			"type"
 #define PROP_NAME_TIME			"time"
 #define PROP_NAME_TIMESTAMP		"timestamp"
 #define PROP_NAME_ACTIVE		"active"
 #define PROP_NAME_MESSAGE		"message"
 #define PROP_NAME_REPEAT		"repeat"
-#define PROP_NAME_SNOOZE		"snooze"
-#define PROP_NAME_NOTIFY_TYPE	"notify_type"
-#define PROP_NAME_SOUND_FILE	"sound_file"
-#define PROP_NAME_SOUND_LOOP	"sound_repeat"
+#define PROP_NAME_NOTIFY_TYPE	"notify-type"
+#define PROP_NAME_SOUND_FILE	"sound-file"
+#define PROP_NAME_SOUND_LOOP	"sound-repeat"
 #define PROP_NAME_COMMAND		"command"
-#define PROP_NAME_NOTIFY_BUBBLE	"notify_bubble"
 
 /* Signal indexes */
 static enum
 {
-  SIGNAL_ALARM,
-  SIGNAL_ERROR,
-  SIGNAL_PLAYER,
-  LAST_SIGNAL
+    SIGNAL_ALARM,
+    SIGNAL_CLEARED,
+    SIGNAL_ERROR,
+    SIGNAL_PLAYER,
+    LAST_SIGNAL
 } AlarmSignal;
 
 /* Signal identifier map */
-static guint alarm_signal[LAST_SIGNAL] = {0, 0, 0};
+static guint alarm_signal[LAST_SIGNAL] = {0, 0, 0, 0};
 
 /* Prototypes for signal handlers */
 static void alarm_alarm (Alarm *alarm);
+static void alarm_cleared (Alarm *alarm);
 static void alarm_error (Alarm *alarm, GError *err);
 static void alarm_player_changed (Alarm *alarm, MediaPlayerState state);
 
-/* For debugging purposes only */
-static void
-dump_list (GSList *list)
-{
-	GSList *l;
-	
-	g_print ("[ ");
-	for (l = list; l; l = l->next) {
-		g_print ("%s", (gchar *)l->data);
-		if (l->next)
-			g_print (", ");
-	}
-	g_print (" ]");
-}
-
 
 /* Initialize the Alarm class */
 static void 
@@ -174,18 +159,17 @@ alarm_class_init (AlarmClass *class)
 {
 	GParamSpec *dir_param;
 	GParamSpec *id_param;
+    GParamSpec *triggered_param;
 	GParamSpec *type_param;
 	GParamSpec *time_param;
 	GParamSpec *timestamp_param;
 	GParamSpec *active_param;
 	GParamSpec *message_param;
 	GParamSpec *repeat_param;
-	GParamSpec *snooze_param;
 	GParamSpec *notify_type_param;
 	GParamSpec *sound_file_param;
 	GParamSpec *sound_loop_param;
 	GParamSpec *command_param;
-	GParamSpec *notify_bubble_param;
 	
 	GObjectClass *g_object_class;
 
@@ -208,7 +192,13 @@ alarm_class_init (AlarmClass *class)
 								  UINT_MAX,	/* max */
 								  0,		/* default */
 								  G_PARAM_CONSTRUCT | G_PARAM_READWRITE);
-	
+
+    triggered_param = g_param_spec_boolean (PROP_NAME_TRIGGERED,
+							                "alarm triggered",
+							                "triggered flag of the alarm",
+                                            FALSE,
+							                G_PARAM_READABLE);
+
 	type_param = g_param_spec_uint (PROP_NAME_TYPE,
 									"alarm type",
 									"type of the alarm",
@@ -253,15 +243,7 @@ alarm_class_init (AlarmClass *class)
 									  ALARM_REPEAT_ALL,		/* max */
 									  ALARM_DEFAULT_REPEAT,	/* default */
 									  G_PARAM_READWRITE);
-	
-	snooze_param = g_param_spec_uint (PROP_NAME_SNOOZE,
-									  "snooze",
-									  "enable snoozing of the alarm",
-									  0,					/* min */
-									  UINT_MAX,				/* max */
-									  ALARM_DEFAULT_SNOOZE,	/* default */
-									  G_PARAM_READWRITE);
-	
+		
 	notify_type_param = g_param_spec_uint (PROP_NAME_NOTIFY_TYPE,
 										"notification type",
 										"what kind of notification should be used",
@@ -286,14 +268,7 @@ alarm_class_init (AlarmClass *class)
 										 "command",
 										 "command to run",
 										 ALARM_DEFAULT_COMMAND,
-										 G_PARAM_READWRITE);
-	
-	notify_bubble_param = g_param_spec_boolean (PROP_NAME_NOTIFY_BUBBLE, 
-												"notification bubble",
-												"whether the alarm should display a notification bubble when triggered",
-												ALARM_DEFAULT_NOTIFY_BUBBLE,
-												G_PARAM_READWRITE);
-	
+										 G_PARAM_READWRITE);	
 	
 	/* override base object methods */
 	g_object_class->set_property = alarm_set_property;
@@ -304,37 +279,47 @@ alarm_class_init (AlarmClass *class)
 	/* install properties */
 	g_object_class_install_property (g_object_class, PROP_DIR, dir_param);
 	g_object_class_install_property (g_object_class, PROP_ID, id_param);
+    g_object_class_install_property (g_object_class, PROP_TRIGGERED, triggered_param);
 	g_object_class_install_property (g_object_class, PROP_TYPE, type_param);
 	g_object_class_install_property (g_object_class, PROP_TIME, time_param);
 	g_object_class_install_property (g_object_class, PROP_TIMESTAMP, timestamp_param);
 	g_object_class_install_property (g_object_class, PROP_ACTIVE, active_param);
 	g_object_class_install_property (g_object_class, PROP_MESSAGE, message_param);
 	g_object_class_install_property (g_object_class, PROP_REPEAT, repeat_param);
-	g_object_class_install_property (g_object_class, PROP_SNOOZE, snooze_param);
 	g_object_class_install_property (g_object_class, PROP_NOTIFY_TYPE, notify_type_param);
 	g_object_class_install_property (g_object_class, PROP_SOUND_FILE, sound_file_param);
 	g_object_class_install_property (g_object_class, PROP_SOUND_LOOP, sound_loop_param);
 	g_object_class_install_property (g_object_class, PROP_COMMAND, command_param);
-	g_object_class_install_property (g_object_class, PROP_NOTIFY_BUBBLE, notify_bubble_param);
 	
 	g_type_class_add_private (class, sizeof (AlarmPrivate));
 	
 	/* set signal handlers */
 	class->alarm = alarm_alarm;
+    class->cleared = alarm_cleared;
 	class->error = alarm_error;
 	class->player_changed = alarm_player_changed;
 
 	/* install signals and default handlers */
 	alarm_signal[SIGNAL_ALARM] = g_signal_new ("alarm",		/* name */
 											   TYPE_ALARM,	/* class type identifier */
-											   G_SIGNAL_RUN_LAST, /* options */
+											   G_SIGNAL_RUN_FIRST, /* options */
 											   G_STRUCT_OFFSET (AlarmClass, alarm), /* handler offset */
 											   NULL, /* accumulator function */
 											   NULL, /* accumulator data */
 											   g_cclosure_marshal_VOID__VOID, /* marshaller */
 											   G_TYPE_NONE, /* type of return value */
 											   0);
-	
+
+    alarm_signal[SIGNAL_CLEARED] = g_signal_new ("cleared",
+                                                 TYPE_ALARM,
+											     G_SIGNAL_RUN_FIRST,
+											     G_STRUCT_OFFSET (AlarmClass, cleared),
+											     NULL,
+											     NULL,
+											     g_cclosure_marshal_VOID__VOID,
+											     G_TYPE_NONE,
+											     0);
+    
 	alarm_signal[SIGNAL_ERROR] = g_signal_new ("error",
 											   TYPE_ALARM,
 											   G_SIGNAL_RUN_LAST,
@@ -494,21 +479,6 @@ alarm_gconf_load (Alarm *alarm)
 		g_object_set (alarm, PROP_NAME_REPEAT, ALARM_DEFAULT_REPEAT, NULL);
 	}
 	
-	/*
-	 * SNOOZE
-	 */
-	key = alarm_gconf_get_full_key (alarm, PROP_NAME_SNOOZE);
-	val = gconf_client_get (client, key, NULL);
-	g_free (key);
-	
-	if (val) {
-		alarm->snooze = gconf_value_get_int (val);
-		gconf_value_free (val);
-	} else {
-		// Not found in GConf, fall back to defaults
-		g_object_set (alarm, PROP_NAME_SNOOZE, ALARM_DEFAULT_SNOOZE, NULL);
-	}
-	
 	/*
 	 * NOTIFY TYPE
 	 */
@@ -571,22 +541,6 @@ alarm_gconf_load (Alarm *alarm)
 		// Not found in GConf, fall back to defaults
 		g_object_set (alarm, PROP_NAME_COMMAND, ALARM_DEFAULT_COMMAND, NULL);
 	}
-	
-	
-	/*
-	 * NOTIFY_BUBBLE
-	 */
-	key = alarm_gconf_get_full_key (alarm, PROP_NAME_NOTIFY_BUBBLE);
-	val = gconf_client_get (client, key, NULL);
-	g_free (key);
-
-	if (val) {
-		alarm->notify_bubble = gconf_value_get_bool (val);
-		gconf_value_free (val);
-	} else {
-		// Not found in GConf, fall back to defaults
-		g_object_set (alarm, PROP_NAME_NOTIFY_BUBBLE, ALARM_DEFAULT_NOTIFY_BUBBLE, NULL);
-	}
 }
 
 static void
@@ -627,17 +581,21 @@ alarm_set_property (GObject *object,
 	GError 		*err = NULL;
 	
 	const gchar	*str;
-	guint		 d, old;
-	AlarmType 	 type, new_type;
-	gboolean	 bool;
+	guint		 d;
+	gboolean	 b;
 	GSList		*list;
 	
 	gchar *key, *tmp;
 
 	alarm = ALARM (object);
 	client = priv->gconf_client;
-	
-	g_debug ("set_property %s", pspec->name);
+
+    // DEBUGGING INFO
+    GValue strval = {0};
+    g_value_init (&strval, G_TYPE_STRING);
+    g_value_transform (value, &strval);
+	g_debug ("Alarm(%p) #%d: set %s=%s", alarm, alarm->id, pspec->name,
+        g_value_get_string(&strval));
 	
 	switch (prop_id) {
 	case PROP_DIR:
@@ -682,6 +640,17 @@ alarm_set_property (GObject *object,
 			alarm_gconf_connect (alarm);
 		}
 		break;
+    case PROP_TRIGGERED:
+        // TODO: Should we map this to a GConf value?
+        //       The only case where this might be useful is where the program
+        //       is restarted while having a triggered alarm so that we could
+        //       re-trigger it when the program starts again...
+        b = g_value_get_boolean (value);
+        
+        if (b != alarm->triggered) {
+            alarm->triggered = b;
+        }
+        
 	case PROP_TYPE:
 		alarm->type = g_value_get_uint (value);
 		
@@ -740,10 +709,10 @@ alarm_set_property (GObject *object,
 		g_free (key);
 		break;
 	case PROP_ACTIVE:
-		bool = alarm->active;
+		b = alarm->active;
 		alarm->active = g_value_get_boolean (value);
 		
-		//g_debug ("[%p] #%d ACTIVE: old=%d new=%d", alarm, alarm->id, bool, alarm->active);
+		//g_debug ("[%p] #%d ACTIVE: old=%d new=%d", alarm, alarm->id, b, alarm->active);
 		if (alarm->active && !alarm_timer_is_started(alarm)) {
 			// Start timer
 			alarm_timer_start (alarm);
@@ -810,21 +779,6 @@ alarm_set_property (GObject *object,
 		g_slist_free (list);
 		g_free (key);
 		
-		break;
-	case PROP_SNOOZE:
-		alarm->snooze = g_value_get_uint (value);
-		
-		key = alarm_gconf_get_full_key (alarm, PROP_NAME_SNOOZE);
-		
-		if (!gconf_client_set_int (client, key, alarm->snooze, &err)) {
-			
-			g_critical ("Could not set %s (gconf): %s", 
-						key, err->message);
-			
-			g_error_free (err);
-		}
-		
-		g_free (key);
 		break;
 	case PROP_NOTIFY_TYPE:
 		alarm->notify_type = g_value_get_uint (value);
@@ -894,22 +848,6 @@ alarm_set_property (GObject *object,
 		
 		g_free (key);
 		
-		break;
-	case PROP_NOTIFY_BUBBLE:
-		bool = alarm->notify_bubble;
-		alarm->notify_bubble = g_value_get_boolean (value);
-		
-		key = alarm_gconf_get_full_key (alarm, PROP_NAME_NOTIFY_BUBBLE);
-		
-		if (!gconf_client_set_bool (client, key, alarm->notify_bubble, &err)) {
-			
-			g_critical ("Could not set %s (gconf): %s", 
-						key, err->message);
-			
-			g_error_free (err);
-		}
-		
-		g_free (key);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -933,6 +871,9 @@ alarm_get_property (GObject *object,
 	case PROP_ID:
 		g_value_set_uint (value, alarm->id);
 		break;
+    case PROP_TRIGGERED:
+        g_value_set_boolean (value, alarm->triggered);
+        break;
 	case PROP_TYPE:
 		g_value_set_uint (value, alarm->type);
 		break;
@@ -951,9 +892,6 @@ alarm_get_property (GObject *object,
 	case PROP_REPEAT:
 		g_value_set_uint (value, alarm->repeat);
 		break;
-	case PROP_SNOOZE:
-		g_value_set_uint (value, alarm->snooze);
-		break;
 	case PROP_NOTIFY_TYPE:
 		g_value_set_uint (value, alarm->notify_type);
 		break;
@@ -966,9 +904,6 @@ alarm_get_property (GObject *object,
 	case PROP_COMMAND:
 		g_value_set_string (value, alarm->command);
 		break;
-	case PROP_NOTIFY_BUBBLE:
-		g_value_set_boolean (value, alarm->notify_bubble);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -987,7 +922,7 @@ alarm_error_quark (void)
 static void
 alarm_error (Alarm *alarm, GError *err)
 {
-	g_debug ("[%p] #%d: alarm_error: #%d: %s", alarm, alarm->id, err->code, err->message);
+	g_debug ("Alarm(%p) #%d: alarm_error: #%d: %s", alarm, alarm->id, err->code, err->message);
 }
 
 void
@@ -1005,7 +940,7 @@ alarm_error_trigger (Alarm *alarm, AlarmErrorCode code, const gchar *msg)
 static void
 alarm_player_changed (Alarm *alarm, MediaPlayerState state)
 {
-	g_debug ("alarm_player_changed (%p, %d)", alarm, state);
+	g_debug ("Alarm(%p) #%d: player_changed to %d", alarm, alarm->id, state);
 }
 
 
@@ -1016,19 +951,17 @@ alarm_player_changed (Alarm *alarm, MediaPlayerState state)
 static void 
 alarm_alarm (Alarm *alarm)
 {
-	g_debug ("[%p] #%d: alarm_alarm", alarm, alarm->id);
-}
+	g_debug ("Alarm(%p) #%d: alarm() DING!", alarm, alarm->id);
 
-void
-alarm_trigger (Alarm *alarm)
-{
-	g_signal_emit (alarm, alarm_signal[SIGNAL_ALARM], 0, NULL);
+    // Clear first, if needed
+    alarm_clear (alarm);
+    
+    // Update triggered flag
+    alarm->triggered = TRUE;
 	
-	/*
-	 * Do we want to repeat this alarm?
-	 */
+	// Do we want to repeat this alarm?
 	if (alarm_should_repeat (alarm)) {
-		g_debug ("alarm_trigger REPEATING");
+		g_debug ("Alarm(%p) #%d: alarm() Repeating...", alarm, alarm->id);
 		alarm_update_timestamp_full (alarm, FALSE);
 	} else {
 		alarm_disable (alarm);
@@ -1037,19 +970,26 @@ alarm_trigger (Alarm *alarm)
 	switch (alarm->notify_type) {
 	case ALARM_NOTIFY_SOUND:
 		// Start sound playback
-		g_debug("[%p] #%d Start player", alarm, alarm->id);
+		g_debug("Alarm(%p) #%d: alarm() Start player", alarm, alarm->id);
 		alarm_player_start (alarm);
 		break;
 	case ALARM_NOTIFY_COMMAND:
 		// Start app
-		g_debug("[%p] #%d Start command", alarm, alarm->id);
+		g_debug("Alarm(%p) #%d: alarm() Start command", alarm, alarm->id);
 		alarm_command_run (alarm);
 		break;
 	default:
-		g_debug ("NOTIFICATION TYPE %d Not yet implemented.", alarm->notify_type);
+		g_warning ("Alarm(%p) #%d: UNKNOWN NOTIFICATION TYPE %d", 
+            alarm, alarm->id, alarm->notify_type);
 	}
 }
 
+void
+alarm_trigger (Alarm *alarm)
+{
+	g_signal_emit (alarm, alarm_signal[SIGNAL_ALARM], 0, NULL);
+}
+
 /*
  * Convenience functions for enabling/disabling the alarm.
  * 
@@ -1093,45 +1033,61 @@ alarm_delete (Alarm *alarm)
 	
 	// Remove configuration
 	key = alarm_gconf_get_dir (alarm);
-	g_debug ("alarm_delete: recursive unset on %s", key);
+	g_debug ("Alarm(%p) #%d: alarm_delete() recursive unset on %s", alarm, alarm->id, key);
 	gconf_client_recursive_unset (client, key, GCONF_UNSET_INCLUDING_SCHEMA_NAMES, NULL);
 	gconf_client_suggest_sync (client, NULL);
 	g_free (key);
 }
 
 /*
- * Snooze the alarm.
+ * Snooze the alarm for a number of seconds.
  */
 void
-alarm_snooze (Alarm *alarm)
+alarm_snooze (Alarm *alarm, guint seconds)
 {
-	// SNOOZE
-	if (alarm->snooze == 0) {
-		alarm_clear (alarm);
-		return;
-	}
-	
-	if (!alarm_is_playing (alarm)) return;
-	
-	g_debug ("alarm_snooze SNOOZING FOR %d minutes", alarm->snooze);
-	
-	time_t now = time (NULL);
+    g_assert (alarm->triggered);
+    
+	g_debug ("Alarm(%p) #%d: snooze() for %d minutes", alarm, alarm->id, seconds / 60);
+
+    // Silence!
+    alarm_clear (alarm);    
+
+    // Remind later
+    time_t now = time (NULL);
 	
 	g_object_set (alarm, 
-				  "timestamp", now + alarm->snooze * 60,
-				  "active", TRUE,
+				  "timestamp", now + seconds,
+				  "active", TRUE,        
 				  NULL);
-	
-	alarm_clear (alarm);
+
+//    alarm_timer_start (alarm);
+}
+
+/**
+ * Alarm cleared signal
+ */
+static void
+alarm_cleared (Alarm *alarm)
+{
+    g_debug ("Alarm(%p) #%d: cleared()", alarm, alarm->id);
+
+    // Update triggered flag
+    alarm->triggered = FALSE;
+
+    // Stop player
+	alarm_player_stop (alarm);
 }
 
 /*
- * Clear the alarm. This will stop any running players.
+ * Clear the alarm. Resets the triggered flag and emits the "cleared" signal.
+ * This will also stop any running players.
  */
 void
 alarm_clear (Alarm *alarm)
 {
-	alarm_player_stop (alarm);
+    if (alarm->triggered) {
+        g_signal_emit (alarm, alarm_signal[SIGNAL_CLEARED], 0, NULL);
+    }
 }
 
 /*
@@ -1160,7 +1116,7 @@ alarm_timer_update (Alarm *alarm)
 		// Remove callback only if we don't intend to repeat the alarm
 		return alarm_should_repeat (alarm);
 	} else if (alarm->timestamp - now <= 10) {
-		g_debug ("[%p] #%d %2d...", alarm, alarm->id, (int)(alarm->timestamp - now));
+		g_debug ("Alarm(%p) #%d: -%2d...", alarm, alarm->id, (int)(alarm->timestamp - now));
 	}
 	
 	// Keep callback
@@ -1172,7 +1128,7 @@ alarm_timer_start (Alarm *alarm)
 {
 	AlarmPrivate *priv = ALARM_PRIVATE(alarm);
 	
-	g_debug ("[%p] #%d timer_start", alarm, alarm->id);
+	g_debug ("Alarm(%p) #%d: timer_start()", alarm, alarm->id);
 	
 	// Remove old timer, if any
 	alarm_timer_remove (alarm);
@@ -1185,7 +1141,7 @@ static gboolean
 alarm_timer_is_started (Alarm *alarm)
 {
 	AlarmPrivate *priv = ALARM_PRIVATE(alarm);
-		
+    
 	return priv->timer_id > 0;
 }
 
@@ -1195,7 +1151,7 @@ alarm_timer_remove (Alarm *alarm)
 	AlarmPrivate *priv = ALARM_PRIVATE(alarm);
 	
 	if (alarm_timer_is_started(alarm)) {
-		g_debug ("#%d timer_remove (%p)", alarm->id, alarm);
+		g_debug ("Alarm(%p) #%d: timer_remove", alarm, alarm->id);
 		
 		g_source_remove (priv->timer_id);
 		
@@ -1342,7 +1298,7 @@ alarm_gconf_dir_changed (GConfClient *client,
 		return;
 	}
 	
-	g_debug ("alarm_gconf changed #%d %p: %s", alarm->id, alarm, name);
+	g_debug ("Alarm(%p) #%d: gconf_dir_changed(): %s", alarm, alarm->id, name);
 	
 	switch (param->param_id) {
 	case PROP_TYPE:
@@ -1364,7 +1320,6 @@ alarm_gconf_dir_changed (GConfClient *client,
 	case PROP_ACTIVE:
 		b = gconf_value_get_bool (entry->value);
 		if (b != alarm->active) {
-			g_debug ("[%p] #%d GCONF-ACTIVE changed from %d to %d", alarm, alarm->id, alarm->active, b);
 			g_object_set (alarm, name, b, NULL);
 		}
 		break;
@@ -1382,11 +1337,6 @@ alarm_gconf_dir_changed (GConfClient *client,
 		
 		g_slist_free (list);
 		break;
-	case PROP_SNOOZE:
-		i = gconf_value_get_int (entry->value);
-		if (i != alarm->snooze)
-			g_object_set (alarm, name, i, NULL);
-		break;
 	case PROP_NOTIFY_TYPE:
 		str = gconf_value_get_string (entry->value);
 		i = alarm_notify_type_from_string (str);
@@ -1408,13 +1358,9 @@ alarm_gconf_dir_changed (GConfClient *client,
 		if (strcmp (str, alarm->command) != 0)
 			g_object_set (alarm, name, str, NULL);
 		break;
-	case PROP_NOTIFY_BUBBLE:
-		b = gconf_value_get_bool (entry->value);
-		if (b != alarm->notify_bubble)
-			g_object_set (alarm, name, b, NULL);
-		break;
 	default:
-		g_warning ("Valid property ID %d not handled!", param->param_id);
+		g_warning ("Alarm(%p) #%d: gconf_dir_changed(): Property ID %d not handled!",
+            alarm, alarm->id, param->param_id);
 		break;
 	}
 	
@@ -1425,10 +1371,10 @@ static void
 alarm_dispose (GObject *object)
 {
 	Alarm *alarm = ALARM (object);
-	AlarmPrivate *priv = ALARM_PRIVATE (object);
+//	AlarmPrivate *priv = ALARM_PRIVATE (object);
 	GObjectClass *parent = (GObjectClass *)alarm_parent_class;
 	
-	g_debug ("alarm_dispose (%p)", alarm);
+	g_debug ("Alarm(%p) #%d: dispose()", alarm, alarm->id);
 	
 	if (parent->dispose)
 		parent->dispose (object);
@@ -1541,144 +1487,29 @@ alarm_gconf_get_dir (Alarm *alarm)
 gchar *
 alarm_gconf_get_full_key (Alarm *alarm, const gchar *key)
 {
+    gchar *gconf_key;
 	gchar *full_key;
 	
 	g_return_val_if_fail (IS_ALARM (alarm), NULL);
 
 	if (!key)
 		return NULL;
-	
-	full_key = g_strdup_printf ("%s/" ALARM_GCONF_DIR_PREFIX "%d/%s", alarm->gconf_dir, alarm->id, key);
 
+    // Replace dashes with underscores
+    gconf_key = g_strdup (key);
+    g_strcanon (gconf_key, "abcdefghijklmnopqrstuvwxyz", '_');
+    
+	full_key = g_strdup_printf ("%s/" ALARM_GCONF_DIR_PREFIX "%d/%s", 
+        alarm->gconf_dir, alarm->id, gconf_key);
+
+    g_free (gconf_key);
+    
 	return full_key;
 }
 
-typedef struct {
-	GObject *object;
-	const gchar *name;
-	gulong handler_id;
-} AlarmBindArg;
-
-static AlarmBindArg *
-alarm_bind_arg_new (GObject *object, const gchar *name, gulong handler_id)
-{
-	AlarmBindArg *arg;
-	
-	arg = g_new (AlarmBindArg, 1);
-	arg->object = object;
-	arg->name = name;
-	arg->handler_id = handler_id;
-	
-	return arg;
-}
-
-static void
-alarm_bind_arg_free (AlarmBindArg *arg)
-{
-	if (!arg)
-		return;
-	
-	g_free ((gpointer)arg);
-}
-
-static void
-alarm_bind_update (GObject *object,
-				   GParamSpec *pspec,
-				   gpointer data)
-{
-	//Alarm *alarm;
-	AlarmBindArg *arg = (AlarmBindArg *)data;
-	gpointer d;
-	
-	/*
-	 * Determine which argument is the alarm
-	 */
-	/*if (IS_ALARM (object)) {
-		 alarm = ALARM (object);
-	} else {
-		alarm = ALARM (arg->object);
-	}
-	
-	g_debug ("alarm_bind_update #%d(%p) [%s] -> %p [%s]", alarm->id, alarm, pspec->name, arg->object, arg->name);
-	*/
-	g_object_get (object, pspec->name, &d, NULL);
-	
-	// Block other signal handler
-	g_signal_handlers_block_matched(arg->object, 
-									G_SIGNAL_MATCH_FUNC,
-									0, 0, NULL, alarm_bind_update, NULL);
-	
-	g_object_set (arg->object, arg->name, d, NULL);
-	
-	g_signal_handlers_unblock_matched(arg->object, 
-									  G_SIGNAL_MATCH_FUNC,
-									  0, 0, NULL, alarm_bind_update, NULL);
-}
-
-/*
- * We would like to disconnect any signal handlers from the alarm
- * when the destination object is finalized. And vice versa.
+/**
+ * Compare two alarms based on ID
  */
-static void
-alarm_bind_weak_notify (gpointer data, GObject *where_the_object_was)
-{
-	AlarmBindArg *arg = (AlarmBindArg *)data;
-	
-	g_assert (arg);
-	g_assert (arg->object);
-	g_assert (arg->handler_id);
-	
-	if (G_IS_OBJECT (arg->object)) {
-		if (g_signal_handler_is_connected (arg->object, arg->handler_id)) {
-			g_signal_handler_disconnect (arg->object, arg->handler_id);
-		}
-	}
-	
-	alarm_bind_arg_free (arg);
-}
-
-/*
- * Binds both ways.
- */
-void
-alarm_bind (Alarm *alarm, 
-			const gchar *prop, 
-			GObject *dest, 
-			const gchar *dest_prop)
-{
-	AlarmBindArg *arg, *obj_arg;
-	GParamSpec *param;
-	gchar *tmp;
-	gulong handler, obj_handler;
-	
-	param = g_object_class_find_property (G_OBJECT_GET_CLASS(alarm), prop);
-	g_assert (param);
-	param = g_object_class_find_property (G_OBJECT_GET_CLASS(dest), dest_prop);
-	g_assert (param);
-	
-//	g_debug ("Bind from %p [%s] -> %p [%s]", alarm, prop, dest, dest_prop);
-	
-	// Alarm -> Object
-	tmp = g_strdup_printf("notify::%s", prop);
-	arg = alarm_bind_arg_new (dest, dest_prop, 0);
-	handler = g_signal_connect (alarm, tmp, G_CALLBACK (alarm_bind_update), arg);
-	g_free(tmp);
-	
-	// Object -> Alarm
-	tmp = g_strdup_printf ("notify::%s", dest_prop);
-	obj_arg = alarm_bind_arg_new (G_OBJECT (alarm), prop, handler);
-	obj_handler = g_signal_connect (dest, tmp, G_CALLBACK (alarm_bind_update), obj_arg);
-	g_free(tmp);
-	
-	arg->handler_id = obj_handler;
-	
-	// Disconnect Object when Alarm is finalized (freed)
-	g_object_weak_ref (G_OBJECT (alarm), alarm_bind_weak_notify, arg);
-	
-	// Disconnect Alarm when Object is finalized (freed)
-	g_object_weak_ref (dest, alarm_bind_weak_notify, obj_arg);
-}
-
 static gint
 alarm_list_item_compare (gconstpointer a, gconstpointer b)
 {
@@ -1720,7 +1551,7 @@ alarm_get_list (const gchar *gconf_dir)
 	GConfClient *client;
 	GSList *dirs = NULL;
 	GSList *l = NULL;
-	gchar *tmp, *dir;
+	gchar *tmp;
 	
 	GList *ret = NULL;
 	Alarm *alarm;
@@ -1737,12 +1568,12 @@ alarm_get_list (const gchar *gconf_dir)
 		
 		id = alarm_gconf_dir_get_id (tmp);
 		if (id >= 0) {
-			g_debug ("alarm_get_list: found VALID %s #%d", tmp, id);
+			g_debug ("Alarm: get_list() found '%s' #%d", tmp, id);
 			
 			alarm = alarm_new (gconf_dir, id);
-			g_debug ("\tref = %d", G_OBJECT (alarm)->ref_count);
+//			g_debug ("\tref = %d", G_OBJECT (alarm)->ref_count);
 			ret = g_list_insert_sorted (ret, alarm, alarm_list_item_compare);
-			g_debug ("\tappend ref = %d", G_OBJECT (alarm)->ref_count);
+//			g_debug ("\tappend ref = %d", G_OBJECT (alarm)->ref_count);
 		}
 		
 		g_free (tmp);
@@ -1764,14 +1595,13 @@ alarm_signal_connect_list (GList *instances,
 {
 	GList *l;
 	Alarm *a;
-	g_debug ("alarm_signal_connect_list");
+	g_debug ("Alarm: signal_connect_list()");
 	for (l = instances; l != NULL; l = l->next) {
 		a = ALARM (l->data);
 		
-		g_debug (" - connecting #%d: %s...", a->id, detailed_signal);
+		g_debug ("\tconnecting Alarm(%p) #%d: %s...", a, a->id, detailed_signal);
 		
-		gboolean ret = g_signal_connect (a, detailed_signal, c_handler, data);
-		g_debug (" = %d", ret);
+		g_signal_connect (a, detailed_signal, c_handler, data);
 	}
 }
 
@@ -1789,7 +1619,7 @@ alarm_player_error_cb (MediaPlayer *player, GError *err, gpointer data)
 	uri = media_player_get_uri (player);
 	msg = g_strdup_printf ("Could not play '%s': %s", uri, err->message);
 	
-	g_critical (msg);
+	g_critical ("%s", msg);
 	
 	/* Emit error signal */
 	alarm_error_trigger (alarm, ALARM_ERROR_PLAY, msg);
@@ -1812,7 +1642,7 @@ alarm_player_state_cb (MediaPlayer *player, MediaPlayerState state, gpointer dat
 	}
 	
 	if (state == MEDIA_PLAYER_STOPPED) {
-		g_debug ("[%p] #%d Freeing media player %p", alarm, alarm->id, player);
+		g_debug ("Alarm(%p) #%d: Freeing media player %p", alarm, alarm->id, player);
 		
 		media_player_free (player);
 		
@@ -1825,7 +1655,7 @@ alarm_player_timeout (gpointer data)
 {
 	Alarm *alarm = ALARM (data);
 	
-	g_debug ("[%p] #%d player_timeout", alarm, alarm->id);
+	g_debug ("Alarm(%p) #%d: player_timeout", alarm, alarm->id);
 	
 	alarm_player_stop (alarm);
 	
@@ -1852,7 +1682,7 @@ alarm_player_start (Alarm *alarm)
 	
 	media_player_start (priv->player);
 	
-	g_debug ("[%p] #%d player_start...", alarm, alarm->id);
+	g_debug ("Alarm(%p) #%d: player_start...", alarm, alarm->id);
 	
 	/*
 	 * Add stop timeout
@@ -1891,7 +1721,7 @@ alarm_command_run (Alarm *alarm)
 		
 		msg = g_strdup_printf ("Could not launch `%s': %s", alarm->command, err->message);
 		
-		g_critical (msg);
+		g_critical ("%s", msg);
 		
 		/* Emit error signal */
 		alarm_error_trigger (alarm, ALARM_ERROR_COMMAND, msg);
@@ -1912,7 +1742,8 @@ alarm_command_run (Alarm *alarm)
 void
 alarm_set_time (Alarm *alarm, guint hour, guint minute, guint second)
 {
-	g_debug ("alarm_set_time (%d:%d:%d)", hour, minute, second);
+	g_debug ("Alarm(%p) #%d: set_time (%d:%d:%d)", alarm, alarm->id, 
+        hour, minute, second);
 	
 	g_object_set (alarm, "time", second + minute * 60 + hour * 60 * 60, NULL);
 }
@@ -1952,7 +1783,8 @@ alarm_set_timestamp (Alarm *alarm, guint hour, guint minute, guint second, gbool
 	AlarmRepeat rep;
 	struct tm *tm;
 	
-	g_debug ("alarm_set_time_full (%d, %d, %d, %d)", hour, minute, second, include_today);
+	g_debug ("Alarm(%p) #%d: set_timestamp (%d, %d, %d, %d)", alarm, alarm->id,
+        hour, minute, second, include_today);
 	
 	time (&now);
 	tm = localtime (&now);
@@ -1965,7 +1797,7 @@ alarm_set_timestamp (Alarm *alarm, guint hour, guint minute, guint second, gbool
 		if (!alarm_time_is_future (tm, hour, minute, second)) {
 			
 			//if (wday < 0) {
-				g_debug("alarm_set_time_full: Alarm is for tomorrow.");
+				g_debug("\tAlarm is for tomorrow.");
 				tm->tm_mday++;
 			/*} else {
 				// wday == tm->tm_wday
@@ -2015,14 +1847,14 @@ alarm_set_timestamp (Alarm *alarm, guint hour, guint minute, guint second, gbool
 		
 		// Calculate distance from now to wday
 		if (wday == 7) {
-			g_debug("alarm_set_time_full: Alarm is in (forced) 1 week.");
+			g_debug("\tAlarm is in (forced) 1 week.");
 			d = 7;
 		} else {
-			g_debug ("d = tm->tm_wday(%d) - wday(%d)", tm->tm_wday, wday);
+//			g_debug ("\td = tm->tm_wday(%d) - wday(%d)", tm->tm_wday, wday);
 			d = alarm_wday_distance (tm->tm_wday, wday);
 		}
 		
-		g_debug ("alarm_set_time_full: Alarm is in %d days.", d);
+		g_debug ("\tAlarm is in %d days.", d);
 		
 		tm->tm_mday += d;
 	}
@@ -2034,10 +1866,10 @@ alarm_set_timestamp (Alarm *alarm, guint hour, guint minute, guint second, gbool
 	// DEBUG:
 	char tmp[512];
 	strftime (tmp, sizeof (tmp), "%c", tm);
-	g_debug ("alarm_set_time_full: Alarm will trigger at %s", tmp);
+	g_debug ("\tAlarm will trigger at %s", tmp);
 	
 	new = mktime (tm);
-	g_debug ("alarm_set_time_full: Setting to %d", new);
+	g_debug ("\tSetting to %d", (gint) new);
 	g_object_set (alarm, "timestamp", new, NULL);
 }
 
@@ -2050,7 +1882,8 @@ alarm_update_timestamp_full (Alarm *alarm, gboolean include_today)
 {
 	if (alarm->type == ALARM_TYPE_CLOCK) {
 		struct tm *tm = alarm_get_time (alarm);
-		g_debug ("update_timestamp_full: %d:%d:%d", tm->tm_hour, tm->tm_min, tm->tm_sec);
+		g_debug ("Alarm(%p) #%d: update_timestamp_full: %d:%d:%d", alarm, alarm->id,
+            tm->tm_hour, tm->tm_min, tm->tm_sec);
 		alarm_set_timestamp (alarm, tm->tm_hour, tm->tm_min, tm->tm_sec, include_today);
 	} else {
 		/* ALARM_TYPE_TIMER */
@@ -2079,11 +1912,15 @@ alarm_get_time (Alarm *alarm)
 	return gmtime (&(alarm->time));
 }
 
-/*
- * Get the remaining alarm time.
- */
 static struct tm tm;
 
+/**
+ * Get the remaining alarm time.
+ *
+ * The return value is a direct pointer to an internal struct and must NOT
+ * be freed. The return value also changes for every call to alarm_get_remain,
+ * so copy it if needed.
+ */
 struct tm *
 alarm_get_remain (Alarm *alarm)
 {
@@ -2102,6 +1939,18 @@ alarm_get_remain (Alarm *alarm)
 	return &tm;
 }
 
+/**
+ * Get the remaining alarm time in seconds
+ */
+time_t
+alarm_get_remain_seconds (Alarm *alarm)
+{
+    time_t now = time (NULL);
+
+    return alarm->timestamp - now;
+}
+
+
 /*
  * AlarmRepeat utilities {{
  */
@@ -2159,6 +2008,58 @@ alarm_should_repeat (Alarm *alarm)
 	return alarm->type == ALARM_TYPE_CLOCK && alarm->repeat != ALARM_REPEAT_NONE;
 }
 
+/*
+ * Create a pretty string representation of AlarmRepeat
+ *
+ * The return value must be freed afterwards
+ */
+gchar *
+alarm_repeat_to_pretty (AlarmRepeat repeat)
+{
+    gchar *str;
+
+    GString *s;
+    struct tm tm;
+    gchar tmp[20];
+    AlarmRepeat r;
+    gint i;
+
+    switch (repeat) {
+        case ALARM_REPEAT_NONE:
+            str = g_strdup (_("Once"));
+            break;
+        case ALARM_REPEAT_WEEKDAYS:
+            str = g_strdup (_("Weekdays"));
+            break;
+        case ALARM_REPEAT_WEEKENDS:
+            str = g_strdup (_("Weekends"));
+            break;
+        case ALARM_REPEAT_ALL:
+            str = g_strdup (_("Every day"));
+            break;
+        default:
+            // Custom repeat, create a list of weekdays
+            s = g_string_new ("");
+	
+	        for (r = ALARM_REPEAT_SUN, i = 0; r <= ALARM_REPEAT_SAT; r = 1 << ++i) {
+		        if (repeat & r) {
+                    tm.tm_wday = i;
+                    strftime (tmp, sizeof(tmp), "%a", &tm);
+                    g_string_append_printf (s, "%s, ", tmp);
+                }
+	        }
+
+            g_string_truncate (s, s->len - 2);
+            
+            str = s->str;
+            
+            g_string_free (s, FALSE);
+            break;
+    }
+
+    return str;
+}
+
 /*
  * }} AlarmRepeat utilities
  */
