@@ -27,12 +27,15 @@
 #include "alarm-applet.h"
 
 void
-preferences_dialog_display (AlarmApplet *applet);
+prefs_init (AlarmApplet *applet);
 
 void
-pref_update_label_show (AlarmApplet *applet);
+prefs_dialog_show (AlarmApplet *applet);
+
+gboolean
+prefs_autostart_get_state (void);
 
 void
-pref_update_label_type (AlarmApplet *applet);
+prefs_autostart_set_state (gboolean state);
 
 #endif /*PREFS_H_*/
